"""
Command-line interface for anki_deck_from_text
"""

import click
from .parse_input import generate_question_answer_dict
from .generate_deck import generate_deck, write_package

CONTEXT_SETTINGS = dict(
    help_option_names=['-h', '--help'],
    show_default=True,
)


@click.command(context_settings=CONTEXT_SETTINGS)
@click.argument("input")
@click.argument("output")
@click.argument("deck_name")
@click.option("--separator", default=" = ", help="Characters that separate the front and back of the cards")
@click.option("--card_model", default="sound",
              help="""Anki card model to build the deck with. Available options are:
              `sound`
              """
              )
def cli(input, output, deck_name, separator, card_model):
    """Generate and Anki deck from annotations on a text file"""

    question_answer_dict = generate_question_answer_dict(
        input=input,
        separator=separator,
    )

    deck = generate_deck(
        question_answer_dict=question_answer_dict,
        deck_name=deck_name,
        card_model=card_model,
    )

    write_package(deck, output)

    click.echo("Finished generating the deck")



"""
Contains the Anki card models to include in the deck.

Implement here your own models by generating a new model id and following an Anki card
structure.
Then add an entry to your new model to the `MODELS` variable. Your model can now be used
in new decks by using the option `card_model` at the command-line.
"""

import genanki
import random

model_id_sound = random.randrange(1 << 30, 1 << 31)
MODEL_SOUND = genanki.Model(
    model_id=model_id_sound,
    name="Sound model",
    fields=[
        {"name": "Question"},
        {"name": "Answer"},
        {"name": "Sound"},
    ],
    templates=[
        {
            "name": "Card to type",
            "qfmt": "{{Question}}<br>{{type:Answer}}",
            "afmt": "{{FrontSide}}<hr id='answer'><br>{{Sound}}",
        },
    ],
    css=""".card {
    font-family: arial;
    font-size: 20px;
    text-align: center;
    color: black;
    background-color: white;
}
"""
)

"""
NEW MODELS IMPLEMENTATION AREA STARTS HERE
"""


"""
NEW MODELS IMPLEMENTATION AREA STOPS HERE
"""

MODELS = {
    "sound": MODEL_SOUND,
}



"""
Contains functions to go from a path to a text file to a dictionary in the format
{question: answer}
"""


def question_answer_split(line, separator):
    line = line.lstrip("- ")
    answer, question = line.split(separator)
    return question, answer


def generate_question_answer_dict(input, separator):
    cards_details = dict()
    with open(input, encoding="utf-8") as FileObj:
        for line in FileObj:
            line = line.strip()
            if not line.startswith("-"):
                continue

            question, answer = question_answer_split(
                line=line,
                separator=separator
            )
            cards_details[question] = answer
    return cards_details



"""
Contains functions that take a dictionary {question: answer} and write an Anki
package (.apkg) with a deck containing the questions and answers.

The deck current available formats are:
- "sound"

Check the models.py file for all models structure and implementation
"""

import random
import genanki
from anki_deck_from_text.models import MODELS


def create_note(question, answer, model_type):
    model = MODELS.get(model_type)
    if model is None:
        raise ValueError("model_type does not exist in the current models.")
    note = genanki.Note(
        model=model,
        fields=[question, answer, ""]
    )
    return note


def create_deck(deck_name):
    deck_id = random.randrange(1 << 30, 1 << 31)
    deck = genanki.Deck(
        deck_id=deck_id,
        name=deck_name,
    )
    return deck


def generate_deck(question_answer_dict, deck_name, card_model):
    deck = create_deck(deck_name=deck_name)

    for key, value in question_answer_dict.items():
        note = create_note(
            question=key,
            answer=value,
            model_type=card_model,
        )
        deck.add_note(note)

    return deck


def write_package(deck, out):
    out = out.removesuffix(".apkg")
    genanki.Package(deck).write_to_file(f"{out}.apkg")
